/*
    The DataBox Libary - developerfriendly working with bytes
    Copyright (C) 2000-2001  Christian Leber

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    e-mail: c_l@bigfoot.com
*/

#include "databox_base64.h"
#include "malloc/plibs_malloc.h"

/* DATABOX TO BASE64 (*CHAR) */
/***********************************************************/
/* 
   read the *input DataBox and return and pointer to an
   malloced memory area with the content of DataBox base64
   encoded

   Returnvalue:
   NULL  = error, nothing changed 
   !NULL = pointer to memory area with the base64 encoded
           content of the input DataBox
*/
char *
DataBox_DataBox_to_base64(DataBox *input)
{
	/* this is the table to encode the data, so we have not to build it
	   every time new */
	char dtable[64] = {65,66,67,68,69,70,71,72,73,74,75,76,77,78,79,80,81,
		82,83,84,85,86,87,88,89,90,97,98,99,100,101,102,103,104,105,106,107,
		108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,48,49,
		50,51,52,53,54,55,56,57,43,47};
	/* the number of 3 byte chunks to workwith */
	int length3;
	int i;
	int inpos; /* actual position in the input string inside the Databox */
	int outpos=0; /* actual position in the output string */
	unsigned char *buff_2;
	char *output;
	
	/* check if the *input is usable */
	if(input==NULL) return NULL;
	if(input->init != 1) return NULL;
	if(input->length==0) return NULL;
	
	/* calculate the number of 3 byte chunks */
	length3 = (input->length / 3);
	if( (input->length % 3) != 0 ) length3 = length3 + 1;

	/* now allocate the memory to return, 4 byte for every
	   3 byte chunk (because of the encoding and 1 byte for the \0 */
	output = (char *)plibs_malloc( (length3 * 4) + 1);

	buff_2 = (unsigned char *)plibs_malloc( (length3 * 3) + 1);
	for(i=0;i<input->length;i++)	{buff_2[i] = input->data[i];}
	for(i=input->length;i<length3*3;i++) {buff_2[i] = 0;} 
	
	for(i=0;i<length3;i++)
	{
		outpos = 4 * i;
		inpos = 3 * i;
		output[outpos] = dtable[buff_2[inpos] >> 2];
		output[outpos + 1] = dtable[((buff_2[inpos] & 3) << 4) | (buff_2[inpos + 1] >> 4)];
		output[outpos + 2] = dtable[((buff_2[inpos + 1] & 0xF) << 2) | (buff_2[inpos + 2] >> 6)];
		output[outpos + 3] = dtable[buff_2[inpos + 2] & 0x3F];
	}

	if (input->length < length3*3)
		{
			output[outpos + 3] = '=';
			if ( (input->length + 1) < length3*3) {output[outpos + 2] = '=';}
	    }

	output[length3*4] = 0;

	plibs_free(buff_2);	
	return output;
}
/***********************************************************/


/* BASE64 (*CHAR) TO DATABOX */
/***********************************************************/
/* 
   read the base64 message in *input and DataBox decodes it
   to the *putput DataBox
   
   Returnvalue:
    0 = all went ok
   -1 = error, nothing changed    FIXME  not right at the moment
*/
int
DataBox_base64_to_DataBox(DataBox *output, char *input)
{
	int c,i,j,length4,pos=0;
	int length_input; /* the length of *input */
	unsigned char dtable[256]={128,128,128,128,128,128,128,128,128,128,128, /* 11 */
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128, /* 17 */
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,62,128,  /* 17 */
		128,128,63,52,53,54,55,56,57,58,59,60,61,128,128,128,0,128,128,128,
		0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,
		25,128,128,128,128,128,128,26,27,28,29,30,31,32,33,34,35,36,37,38,
		39,40,41,42,43,44,45,46,47,48,49,50,51,128,128,128,128,128,128,128,
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
		128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,128,
		128,128,128,128,128,128,128};
	unsigned char a[4], b[4];

	/* proof if it's probable that *output if usable */
	if(output == NULL) return -1;
	if(output->init != 1) return -1;

	length_input = strlen(input);
	/* an base64 input with no length makes no sense */
	if(length_input == 0) return -1;

	/* now we test if the number of base64 input chars is 4*n (n natural number) */
	length4 = length_input / 4;
	if(length4 * 4 != length_input) return -1;

	/* check if there is space for length4*3 bytes */
	DataBox_careless_ensure_size(output,length4*3);

	/* that would be the length of the output if all bytes were "used" */
	output->length = length4 * 3;

	for(j=0;j<length4;j++)
	{
		for (i = 0; i < 4; i++)
		{
			c = input[(j*4) + i];
			if (dtable[c] & 0x80)
			{
				//printf("%s\n","invalid character, exiting base64_to_DataBox");
				output->length=0;
				return 1;
			}
			a[i] = c;
			b[i] = dtable[c];
		}
		output->data[pos++] = (b[0] << 2) | (b[1] >> 4);
		output->data[pos++] = (b[1] << 4) | (b[2] >> 2);
		output->data[pos++] = (b[2] << 6) | b[3];
		i = a[2] == '=' ? 1 : (a[3] == '=' ? 2 : 3);
	}
	
	if(a[3]=='=') output->length = output->length - 1;
	if(a[2]=='=') output->length = output->length - 1;

	return 0;
}
/***********************************************************/
