/*
    The DataBox Libary - developerfriendly working with bytes
    Copyright (C) 2000-2001  Christian Leber

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    e-mail: c_l@bigfoot.com
*/

#include <stdlib.h>
#include "databox_core.h"
#include "../plibs_types.h"

/* ok, that is not a nice solution, but it takes just 1024 byte
 * when crc32 stuff was used, i consider this not as a problem */
static unsigned long *_databox_crc32_table=NULL;

/* Definitions for internal use */
static void _DataBox_crc32_init(void);
unsigned long _DataBox_crc32_calc(const unsigned char *str,plibs_u32 length);


/* ADD CRC32 CHECKSUM */
/***********************************************************/
/* 
   calculate and append a crc32 checksum to the *thisb DataBox
   
   Returnvalue:
    0 = ok 
   -1 = error
*/
int
DataBox_crc32_add(DataBox *thisb)
{
	plibs_u32 crc32;
	if(thisb==NULL) return -1;
	if(thisb->init != 1) return -1;
	if(thisb->length<=0) return -1;

	crc32 = _DataBox_crc32_calc(thisb->data,thisb->length);
	if(DataBox_resize(thisb,thisb->length+4) < 0 ) return -1;
		
	thisb->data[thisb->length]=(crc32 & 0xFF000000) >> 24;
	thisb->data[thisb->length+1]=(crc32 & 0x00FF0000) >> 16;
	thisb->data[thisb->length+2]=(crc32 & 0x0000FF00) >>  8;
	thisb->data[thisb->length+3]=(crc32 & 0x000000FF);

	thisb->length = thisb->length + 4;

	return 0;
}
/***********************************************************/


/* CHECK AND REMOVE CRC32 CHECKSUM */
/***********************************************************/
/* 
   check if the crc32 checksum is ok and remove it if is ok
   
   Returnvalue:
    0 = ok 
   -1 = error
   -2 = checksum not correct, nothing changed
*/
int
DataBox_crc32_checkandremove(DataBox *thisb)
{
	/* this is the crc32 sum from the DataBox */
	plibs_u32 toproof;
	/* calculated crc32 sum from the rest */
	plibs_u32 crc32;

	if(thisb==NULL) return -1;
	if(thisb->init != 1) return -1;
	if(thisb->length<=4) return -1;

	toproof = thisb->data[thisb->length - 4];
	toproof = (toproof << 8) + thisb->data[thisb->length - 3];
	toproof = (toproof << 8) + thisb->data[thisb->length - 2];
	toproof = (toproof << 8) + thisb->data[thisb->length - 1];

	crc32 = _DataBox_crc32_calc(thisb->data,thisb->length-4);

	if(crc32 == toproof)
	{
		thisb->length = thisb->length - 4;
		return 0;
	}
	else return -2;
}
/***********************************************************/


/* stuff vor internal use */

/* build the crc table */
static void _DataBox_crc32_init(void)
{
	register unsigned long crc;
	unsigned long polynom;
	register int i, j;

    /* FIXME */
	_databox_crc32_table = (unsigned long *)malloc(sizeof(unsigned long)*256);
	polynom = 0xEDB88320L;

	for (i = 0; i < 256; i++)
	{
		crc = i;
		for (j = 8; j > 0; j--)
		{
			if (crc & 1)
				crc = (crc >> 1) ^ polynom;
			else
				crc >>= 1;
		}
		_databox_crc32_table[i] = crc;
	}
}


/* calculate the crc value */
unsigned long _DataBox_crc32_calc(const unsigned char *str, plibs_u32 length)  
{
	register unsigned long crc;
	register plibs_u32 c,i;

	if (_databox_crc32_table==NULL) _DataBox_crc32_init();
 
	crc = 0xFFFFFFFF;
	for(i=0;i<length;i++)
	{
		c = (unsigned int)str[i];
		crc = ((crc>>8) & 0x00FFFFFF) ^ _databox_crc32_table[ (crc^c) & 0xFF ];
	}
	return( crc^0xFFFFFFFF );
}
 
