
#include "databox_crypto_rsa.h"
#include "databox_mpz.h"
#include "../config.h"


/* #######  RSA  ####### */

/*
Return Values:
-1 error, do not trust *workwith, but it should be unchanged
*/


int DataBox_rsa2_encrypt(DataBox *workwith, DataBoxRSA *input_key)
{
	mpz_t mpztmp1;
	mpz_t mpztmp2;
	
	/* proof if there is a key */
	if(input_key==NULL) return -1;
	if(input_key->init != 1) return -1;

	/* FIXME */
	/* if mode is 1, then it is a public key */
	if(input_key->mode != 1) return -1;
	
	/* proof if workwith is usable */
	if(workwith==NULL) return -1;
	if(workwith->init != 1) return -1;
	
	/* proof that workwith is not empty */
	if(workwith->length==0) return -1;
	
	DataBox_DataBox_to_mpz(mpztmp1, workwith);

	/* proof if mpztmp1 > 0 */
	if(mpz_cmp_ui(mpztmp1,0)<=0) return -1;

	/* proof that input < input_key->n */
	if(mpz_cmp(input_key->n,mpztmp1)<=0) return -1;

	mpz_init(mpztmp2);

	/* decrypt */
	mpz_powm(mpztmp2,mpztmp1,input_key->e,input_key->n);

	if(DataBox_mpz_to_DataBox(workwith,mpztmp2) < 0) return -1;
	
	return 0;
}

int DataBox_rsa2_decrypt(DataBox *workwith, DataBoxRSA *input_key)
{
	mpz_t mpztmp1;
	mpz_t mpztmp2;

	/* proof if there is a key  FIXME */
	if(input_key==NULL) return -1;

	/* proof if workwith is usable */
	if(workwith==NULL) return -1;
	if(!workwith->init) return -1;
	
	/* proof that workwith is not empty */
	if(workwith->length==0) return -1;
	
	DataBox_DataBox_to_mpz(mpztmp1, workwith);

	/* proof if mpztmp1 > 0 */
	if(mpz_cmp_ui(mpztmp1,0)<=0) return -1;

	mpz_init(mpztmp2);

	/* decrypt */
	mpz_powm(mpztmp2,mpztmp1,input_key->d,input_key->n);

	if(DataBox_mpz_to_DataBox(workwith, mpztmp2) < 0) return -1;
	
	return 0;
}




/****************************
 *
 * OLD
 *
 * **************************/

/* #######  RSA  ####### */

/*
Return Values:
-1 error, do not trust *workwith, but it should be unchanged
*/

int DataBox_rsa_encrypt(DataBox *workwith, clrsa_public *input_key)
{
	mpz_t mpztmp1;
	mpz_t mpztmp2;
	
	/* proof if there is a key  FIXME */
	if(input_key==NULL) return -1;

	/* proof if workwith is usable */
	if(workwith==NULL) return -1;
	if(!workwith->init) return -1;
	
	/* proof that workwith is not empty */
	if(workwith->length==0) return -1;
	
	DataBox_DataBox_to_mpz(mpztmp1, workwith);

	/* proof if mpztmp1 > 0 */
	if(mpz_cmp_ui(mpztmp1,0)<=0) return -1;

	/* proof that input < input_key->n */
	if(mpz_cmp(input_key->n,mpztmp1)<=0) return -1;

	mpz_init(mpztmp2);

	/* decrypt */
	mpz_powm(mpztmp2,mpztmp1,input_key->e,input_key->n);

	if(DataBox_mpz_to_DataBox(workwith,mpztmp2) < 0) return -1;
	
	return 0;
}

int DataBox_rsa_decrypt(DataBox *workwith, clrsa_privat *input_key)
{
	mpz_t mpztmp1;
	mpz_t mpztmp2;

	/* proof if there is a key  FIXME */
	if(input_key==NULL) return -1;

	/* proof if workwith is usable */
	if(workwith==NULL) return -1;
	if(!workwith->init) return -1;
	
	/* proof that workwith is not empty */
	if(workwith->length==0) return -1;
	
	DataBox_DataBox_to_mpz(mpztmp1, workwith);

	/* proof if mpztmp1 > 0 */
	if(mpz_cmp_ui(mpztmp1,0)<=0) return -1;

	mpz_init(mpztmp2);

	/* decrypt */
	mpz_powm(mpztmp2,mpztmp1,input_key->d,input_key->n);

	if(DataBox_mpz_to_DataBox(workwith, mpztmp2) < 0) return -1;
	
	return 0;
}

