/*
    databox_hash.c - hash functionses for the DataBox libary
    Copyright (C) 2000-2001  Christian Leber

    This file is part of the DataBox crypto libary.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    e-mail: c_l@bigfoot.com
*/

#include <stdlib.h>
#include "databox_hash.h"

/* CREATE MD5 HASH FOR DATABOX */
/***********************************************************/
/* 
   reads *input and creates an MD5 HASH in *md5_output
   the size of the output is 16 byte
   
   *input       will stay untouched
   *md5_output  will be destroyed, and filled up with the MD5
   
   if *input is empty an error (-1) will be returned 
   
   Returnvalue:
    0 = all went ok
   -1 = error
*/
int
DataBox_hash_md5_normal(DataBox *md5_output, DataBox *input)
{
	/* check if the DataBoxes are usable */
	if(md5_output==NULL) return -1;
	if(md5_output->init != 1) return -1;
	if(input==NULL) return -1;
	if(input->init != 1) return -1;
	if(input->length < 1) return -1;

	/* ok, we need 16 byte for the hash */
	if(DataBox_careless_ensure_size(md5_output,16) < 0) return -1;
	/* now there are at least 16 bytes ready beginning
	   at md5_output->data */
	
	md5_for_DataBox(md5_output->data,input->data,input->length);
	
	/* do there are now 16 bytes in *md5_output */
	md5_output->length=16;
	
	return 0;
}
/***********************************************************/

