/*
    databox_mpz.h - convert DataBox zu mpz and reverse
    Copyright (C) 2000-2001  Christian Leber

    This file is part of the DataBox crypto libary.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    e-mail: c_l@bigfoot.com
*/

#ifndef PLIBS_DATABOX_MPZ_H_
#define PLIBS_DATABOX_MPZ_H_

#include "../databox/databox_core.h"
/* just includes gmp.h */
#include "../config/zy_gmp.h"


/* MPZ TO DATABOX */
/***********************************************************/
/* 
   reads the mpz_t input and converts it to a DataBox *output

   input    will stay untouched
   *output  will be destroyed, and filled up with the output
   
   if input is <= 0 then an error (1) will be returned 
   
   Returnvalue:
    0 = all went ok
   -1 = error, nothing changed 
*/
int
DataBox_mpz_to_DataBox(DataBox *output, mpz_t input);


/* DATABOX TO MPZ */
/***********************************************************/
/* 
   reads the DataBox *input and converts it to a mpz_t output

   *input    will stay untouched
   output    will be changed
   
   If the length of the *input is < 1 an error will be
   returned
   
   Returnvalue:
    0 = all went ok
   -1 = error, nothing changed 
*/
int
DataBox_DataBox_to_mpz(mpz_t output, DataBox *input);

#endif /* PLIBS_DATABOX_MPZ_H_ */
