/*
    coridora.c - this is a filter you may place between IRC-Server and your
	favourite client
    Copyright (C) 2001-2003 Christian Leber

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

    e-mail: christian@leber.de
*/

#include "../config.h"

#include <stdio.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <string.h>
#include <unistd.h>
#include <stdlib.h>
#include <strings.h>
#include <sys/select.h>
#include <signal.h>
#include <errno.h>
#include <stdarg.h>
#include "cmdarg.h"
#include "tcpstuff.h"
#include "databox_crypto/databox_crypto.h"
#include "dstruct.h"

#define max(a,b)	((a) > (b) ? (a) : (b))

/**************************************************************************/
/* some static declarations                                               */

/* for the stuff from the client to the server */
#define SOCKBUF_SIZE 1024
char SOCKBUF[SOCKBUF_SIZE];
int SOCKBUF_POS=0;

/* this struct holds the informations from the server that will be send to
 * client */
static struct_tcpstuff *SERV;
/* this struct is especially important, it is specified in dstruct.h */
static node_s_list *DS_s;
/* this is the SOCKET for the connection to the client */
int connfd; 

char debug_to_file=0;

/**************************************************************************/
/* function declarations                                                  */

int SERVbuf_to_clientput(void);
void put_server_to_client(char *input, int input_length);

void socket_to_sockbuf(void);
int sockbuf_to_put_to_server(void);
void put_to_server(char *input);

int printf_to_tcpSend(int socket, int nr, char *format, ...);

/**************************************************************************/

int SERVbuf_to_clientput(void)
{
	int a_pos=0,e_pos=0;

	/* while we are not at the end of messages */
	while(e_pos < SERV->SOCKBUF_POS-1)
	{
		if(SERV->SOCKBUF[e_pos]=='\r'&&SERV->SOCKBUF[e_pos+1]=='\n')
		{
			if(e_pos-a_pos>0)
			{
				SERV->SOCKBUF[e_pos]=0;
				put_server_to_client(SERV->SOCKBUF+a_pos,e_pos-a_pos);
			}
			a_pos=e_pos+2;
			e_pos++;
		}
		e_pos++;
	}

	SERV->SOCKBUF_POS-=a_pos;
	if(SERV->SOCKBUF_POS>0)
		memmove(SERV->SOCKBUF,SERV->SOCKBUF+a_pos,SERV->SOCKBUF_POS);
	return 0;
}


void put_server_to_client(char *input, int input_length)
{
	char *buf_1,*buf_2,*to_nick;
	char fname[256], st1[4097], st2[4097];
	int len;
	FILE *fp, *fstream;
	node_nick_list *np;
	DataBox *DBtemp_1=NULL,*DBinput=NULL;  /* here we put the input message */
	DataBoxBLOWFISH *blowfish_key=NULL;
	clrsa_privat rsa_privat;
	
	len=input_length+1;

	/* DEBUG */
	if(debug_to_file==1)
	{
		fp = fopen("log.coridoras","a");
		fprintf(fp,"|<-1|%s||\n",input);
		fclose(fp);
	}

	/* look if it's a PRIVMSG */
	buf_1 = (char *)alloca(len);
	buf_2 = (char *)alloca(len);
	to_nick = (char *)alloca(len);

	if(sscanf(input,":%50[^!]!%*s PRIVMSG %*s :@%500s",buf_1,buf_2)==2)
	{
		DBinput = DataBox_init(0);
		if(DataBox_base64_to_DataBox(DBinput,buf_2) < 0)
		{
			printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
			"!! error while decoding (base64) message",buf_1,DS_s->NICKNAME);
			DataBox_destroy(DBinput);
			return;
		}
		if(DBinput->data[0] == 21)
		{
			DataBox_delete_first_uchar(DBinput);
			snprintf(fname,255,"%s/%s.privat.rsa",DS_s->key_directory_privat,DS_s->NICKNAME);
			fstream=fopen(fname,"r");
			if(fstream == NULL)
			{
				printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
					"!! the privat key file(%s) could not be found",
					buf_1,DS_s->NICKNAME,fname);
				return;
			}
			if(fscanf(fstream,"%*s\n%4096s\n%4096s",st1,st2)!=2)
			{
				printf("the privat key file is not in the right format\n");
				return;
			}
			fclose(fstream);

			mpz_init(rsa_privat.n);
			mpz_init(rsa_privat.d);
			mpz_set_str(rsa_privat.n,st1,10);
			mpz_set_str(rsa_privat.d,st2,10);

			if(DataBox_rsa_decrypt(DBinput, &rsa_privat) < 0)
				printf("PROBLEM WITH RSA DECRYPT\n");

			blowfish_key = DataBox_blowfish_init();
			if(DataBox_DataBox_to_DataBoxBLOWFISH(blowfish_key,DBinput) < 0)
				printf("DataBox_DataBox_to_DS_blowfish failed\n");

			/* look up the nickname in the DS or add it */
			np = DS_nick_lookup(DS_s,buf_1);

			if(np==NULL)
			{
				np = DS_nick_install(DS_s,buf_1);
			}
			np->blowfish_query_recv = blowfish_key;

			printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
				"!! received blowfish key",buf_1,DS_s->NICKNAME);
		}
		if(DBinput->data[0] == 31)
		{
		        DataBox_delete_first_uchar(DBinput);
        		np = DS_nick_lookup(DS_s, buf_1);
			if(np==NULL)
			{
				printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
					"!! ERROR: we have no blowfish key",buf_1,DS_s->NICKNAME);
				return;
			}
			if(np->blowfish_query_recv==NULL)
			{
				printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
					"!! ERROR: we have no blowfish key",buf_1,DS_s->NICKNAME);
				return;
			}
			if(DataBox_blowfish_decrypt(DBinput, np->blowfish_query_recv)<0)
				printf("DataBox_blowfish_decrypt: error\n");
			DBtemp_1 = DataBox_init(0);
			DataBox_tdecompress1(DBtemp_1, DBinput);
			printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
				"%s",buf_1,DS_s->NICKNAME,(char *)DataBox_DataBox_to_ucharstr(DBtemp_1));
		}
	}
	else if(sscanf(input,":%50[^!]!%*s PRIVMSG %50s :!!%500[^\n]",buf_1,to_nick,buf_2)==3)
	{
		if(strcmp(DS_s->NICKNAME,to_nick)==0)
			printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
				"!! Warning, got: \"!!%s\"",buf_1,DS_s->NICKNAME,buf_2);
		else
			printf_to_tcpSend(connfd,2,"%s",input);
	}
	else if(sscanf(input,":%50[^!]!%*s PRIVMSG %50s :%500[^\n]",buf_1,to_nick,buf_2)==3)
	{
		if(to_nick[0]=='#' || to_nick[0]=='+' || to_nick[0]=='!' || to_nick[0]=='&')
		{
			printf_to_tcpSend(connfd,2,"%s",input);
		}
		else
		{
			np = DS_nick_lookup(DS_s,buf_1);
			if(np==NULL)
			{
				np = DS_nick_install(DS_s,buf_1);
			}
			if(np->query_mode==0)
			{
				np->query_mode=1;
				printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
					"!! !! Warning, this channel is not encrypted !! !!",buf_1,DS_s->NICKNAME);
				printf_to_tcpSend(connfd,2,"%s",input);
			}
			else
			{
				printf_to_tcpSend(connfd,2,"%s",input);
			}
		}
	}
	else if(sscanf(input,":%50[^!]!%*s NICK :%500s",buf_1,buf_2)==2)
	{
		if(strcmp(DS_s->NICKNAME,buf_1)==0)
		{
				free(DS_s->NICKNAME);
				DS_s->NICKNAME = strdup(buf_2);
		}
		else
		{
			printf_to_tcpSend(connfd,2,"%s",input);
		}
	}
	else if(sscanf(input,":%*s 433 * %50s :%*s",buf_1)==1)
	{
		/* this happens when we try to use a NICKNAME that is allready in use */
		/* but only at the beginning of the connection, therefore the "*", because
		 * we have no NICKNAME at this point */
		if(strcmp(DS_s->NICKNAME,buf_1)==0)
		{
			free(DS_s->NICKNAME);
			DS_s->NICKNAME=NULL;
		}
		printf_to_tcpSend(connfd,2,"%s",input);
	}
	else
	{
		printf_to_tcpSend(connfd,2,"%s",input);
	}
}


/* #####
 * now the part that takes the stuff from the client and
 * puts it to the irc server
 */

/* reads all data avaible on the socket from the client and put it in the
   SOCKBUF */
void socket_to_sockbuf(void)
{
	int length;
	
	/* check if there is space for additional bytes */
	if(SOCKBUF_SIZE <= SOCKBUF_POS) return;

	/* try to read as many bytes as room is avaible */
	length = read(connfd,SOCKBUF+SOCKBUF_POS,
			SOCKBUF_SIZE-SOCKBUF_POS-1);
	/* if the return value of read is -1 it means that there is
		something really wrong with this socket, we'll exit */
	if( length == -1)
	{
		printf("fatal error: can't read from the socket!!\nbye\n");
		exit(1);
	}
	/* FIXME what is if it is 0 ? */

	SOCKBUF[SOCKBUF_POS+length] = 0;
	SOCKBUF_POS += length;
}


/* splits SOCKBUF in lines and gives them to put_to_server(char *input) */
int sockbuf_to_put_to_server(void)
{
	int len;
	char *buf,*pos;
	
	pos = strstr(SOCKBUF,"\r\n");
	if(pos != NULL)
	{
		len = pos - SOCKBUF;
		SOCKBUF[len] = 0; /* "\0" */
		put_to_server(SOCKBUF);
		memcpy(SOCKBUF,pos+2,SOCKBUF_POS-len-2);
		SOCKBUF_POS -= (len+2);
		SOCKBUF[SOCKBUF_POS] = 0;
		return 0;
	}
	else
	{
		pos = strstr(SOCKBUF,"\n");
		if(pos != NULL)
		{
			len = pos - SOCKBUF;
			SOCKBUF[len] = 0; /* "\0" */
			put_to_server(buf);
			memcpy(SOCKBUF,pos+1,SOCKBUF_POS-len-1);
			SOCKBUF_POS -= (len+1);
			SOCKBUF[SOCKBUF_POS] = 0;
			return 0;
		}
	}
	return -1;
}

void put_to_server(char *input)
{
	char *buf;
	int len;
	char *buf_1,*buf_2,*buf_3,*output;
	char fname[256];
	clrsa_public rsa_public;
	DataBoxBLOWFISH *key_create=NULL;
	DataBox *DB_temp_1=NULL;
	DataBox *DB_temp_2=NULL;
	FILE *fp;
	char st1[4097],st2[4097]; /* FIXME */
	node_nick_list *np;
	FILE *fstream;
	len=strlen(input);
	
	/* DEBUG */
	if(debug_to_file==1)
	{
		fp = fopen("log.coridoras","a");
		fprintf(fp,"|<-2|%s||\n",input);
		fclose(fp);
	}

	if(len>1000)
	{
		printf("MSG TOOOO LOOONNG! - don't forward\n");
		return;
	}

	buf = (char *)alloca(1200);
	buf_1 = (char *)alloca(1025);
	buf_2 = (char *)alloca(1025);
	buf_3 = (char *)alloca(1025);

	if(sscanf(input,"PRIVMSG %512s :%512[^\n]",buf_1,buf_2)==2)
	{
		if(buf_2[0]=='!' && buf_2[1]=='!')
		{
			/* a command */
			if(buf_2[2]=='s')
			{
				/* send key to buf_3 */
				snprintf(fname,255,"publickeys/%s.public.rsa",buf_1);
				fstream=fopen(fname,"r");
				/* check if it was successfull to open this file */
				if(fstream == NULL)
				{
					printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG ijut :!!"\
					" \"!!s\" sorry, be we have no public key for this nickname",buf_1);
					return;
				}
				if(fscanf(fstream,"%*s\n%4096s\n%4096s",st1,st2)!=2)
				{
					printf("%s\n","publickeyformat brocken");
					fclose(fstream);
					return;
				}
				fclose(fstream);

				/* LOOK */
				np=DS_nick_lookup(DS_s,buf_1);
				if(np==NULL)
					np=DS_nick_install(DS_s,buf_1);
				
				mpz_init(rsa_public.n);
				mpz_init(rsa_public.e);
				mpz_set_str(rsa_public.n,st1,10);
				mpz_set_str(rsa_public.e,st2,10);

				/* ok, the public key is loaded
				 * and we crate the symmetric key */
				key_create = DataBox_blowfish_init();
				DataBox_blowfish_keygen(key_create, 16);
				
				np->blowfish_query_send = key_create;
				DB_temp_1 = DataBox_init(0);
				DataBox_DataBoxBLOWFISH_to_DataBox(DB_temp_1, key_create);
				
				//if(DataBox_crc32_add(DB_temp_1) < 0)
				//{
					/* uh, oh */
				//	printf("%s\n","SIT_create_symkeymsg: DataBox_crc32_add failed");
				//}
				DataBox_rsa_encrypt(DB_temp_1, &rsa_public);
				DataBox_insert_first_uchar(DB_temp_1, 21);
				output = DataBox_DataBox_to_base64(DB_temp_1);
				DataBox_destroy(DB_temp_1);
				printf_to_tcpSend(SERV->SOCKET,1,"PRIVMSG %s :@%s",
								buf_1,output);
			}
			else if(strcmp(buf_2+2,"version")==0)
			{
				printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG ijut :!!"\
					" You are using coridoras %s",buf_1,VERSION);
			}
			else
			{
				/* create message with the options */
				printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG ijut :!!"\
					" Help:",buf_1);
				printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG ijut :!!"\
					" \"!!s\" sends a new blowfish key to the other side",buf_1);
				printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG ijut :!!"\
					" \"!!version\" shows the version you use",buf_1);
			}
		}
		else
		{
			if(buf_1[0]=='#' || buf_1[0]=='+' || buf_1[0]=='!' || buf_1[0]=='&')
			{
				/* channel message */
				printf_to_tcpSend(SERV->SOCKET,1,"%s",input);
			}
			else
			{
				/* query/privat message */
				np = DS_nick_lookup(DS_s,buf_1);
				if(np==NULL)
				{
					np = DS_nick_install(DS_s,buf_1);
					np->query_mode=1;
					printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
						"!! !! Warning, this channel is not encrypted !! !!",buf_1,DS_s->NICKNAME);
					printf_to_tcpSend(SERV->SOCKET,1,"%s",input);
				}
				else
				{
					if(np->blowfish_query_send!=NULL)
					{
						/* so we allready have a blowfish key */
						DB_temp_1 = DataBox_init(0);
						DataBox_ucharstr_to_DataBox(DB_temp_1, (unsigned char *)buf_2);
						
						DB_temp_2 = DataBox_init(0);
						DataBox_tcompress1(DB_temp_2, DB_temp_1);
						DataBox_destroy(DB_temp_1);

						DataBox_blowfish_encrypt(DB_temp_2, np->blowfish_query_send);
						DataBox_insert_first_uchar(DB_temp_2,31);
					
						output = DataBox_DataBox_to_base64(DB_temp_2);
						printf_to_tcpSend(SERV->SOCKET,1,"PRIVMSG %s :@%s",
							buf_1,output);
						DataBox_destroy(DB_temp_2);
						free(output);
					}
					else
					{
						if(np->query_mode==0)
						{
							np->query_mode=1;
							printf_to_tcpSend(connfd,2,":%s!coridoras@coridoras PRIVMSG %s :"\
								"!! !! Warning, this channel is not encrypted !! !!",buf_1,DS_s->NICKNAME);
						}
						printf_to_tcpSend(SERV->SOCKET,1,"%s",input);
					}
				}
			}
		}
	}
	else if(sscanf(input,"NICK %512s",buf_1)==1)
	{
		/* in this way the local data structure is just changed at the
		 * beginning */
		if(DS_s->NICKNAME==NULL) DS_s->NICKNAME=strdup(buf_1);
		printf_to_tcpSend(SERV->SOCKET,1,"%s",input);
	}
	else if(sscanf(input,"QUIT :%512s",buf_1)==1)
	{
		printf_to_tcpSend(SERV->SOCKET,1,"%s",input);
		printf("ok, then we'll exit too\n");
		exit(0);
	}
	else			
	{
		printf_to_tcpSend(SERV->SOCKET,1,"%s",input);
	}
}


int printf_to_tcpSend(int socket, int nr, char *format, ...)
{
	char *buff1; /* general buffer... you know that :)) */
	int retval=0; /* return value from snprintf */
	va_list arg_list;
	FILE *fp;
	
	buff1 = (char *)alloca(513);

	va_start(arg_list, format);
	retval = vsnprintf(buff1,511,format,arg_list);
	va_end(arg_list);
	if(retval < 0 || retval >510)
	{
		printf("ERROR: string to big, will not be sended");
		return -1;
	}

	/* DEBUG */
	if(debug_to_file==1)
	{
		fp = fopen("log.coridoras","a");
		fprintf(fp,"|->%d|%s||\n",nr,buff1);
		fclose(fp);
	}

	buff1[retval]='\r';
	retval++;
	buff1[retval]='\n';
	retval++;
	buff1[retval]=0;

	if(send(socket,buff1,retval,0)!=retval)
	{
		printf("%s\n","ERROR: SIT_tcpSend, the number\
		of bytes to send and the return value of send are NOT the same");
		return -1;
	}
	return 0;
}


/* this function will be started if SIGINT is sended to sit */
void SIT_quitsignal(int signo)
{
	close(connfd);
	close(SERV->SOCKET);
	printf("received SIGINT\n");
	exit(0);
}

int main(int argc, char **argv)
{
	const int on;
	int netfd;      /* the socket for bind() */
	struct sockaddr_in servaddr, cliaddr;
#if defined __CYGWIN__ || __EMX__ /* cygwin, EMX */
	int clilen;
#else                   /* cygwin, EMX */
	socklen_t clilen;
#endif                  /* cygwin, EMX */
	char buff[530]; /* for the stuff from stdin, to try out how the
			client reacts on messages */
	fd_set rset; 	/* for select */
	int maxfdp1;	/* for select */
	Cmdline *cmd;

	SERV = (struct_tcpstuff *)malloc(sizeof(*SERV));
	DS_s = (node_s_list *)malloc(sizeof(*DS_s));
	DS_s->DS_nick[0]=NULL;
	DS_s->NICKNAME=NULL;
	cmd = parseCmdline(argc,argv);

	/* first we check if an option for help was given */
	if(cmd->helpP!=0){usage();}
	if(cmd->helplongP!=0){usage();} 

	if(cmd->versionP!=0)
	{
		printf("This is %s\n",PACKAGE_STRING);
		exit(0);
	}

	if(cmd->debug_to_fileP!=0) debug_to_file=1;

	if(getenv("HOME")!=NULL)
	{
		DS_s->key_directory_privat = (char *)malloc(strlen(getenv("HOME"))+30);
		sprintf(DS_s->key_directory_privat,"%s/.coridoras/PRIVAT",getenv("HOME"));
		printf("Directory for private keys: %s\n",DS_s->key_directory_privat);
	}
	else
	{
		printf("You have no $HOME set, please specify the directory where your PRIVAT directory is with the -kd option\n");
		exit(0);
	}
	
	Tcpstuff_init(SERV,cmd->server,(unsigned short int)cmd->server_port);
	
	printf("will connect to %s:%u\n",SERV->SERVER,SERV->PORT);

	 /* signal handler */
	signal(SIGINT, SIT_quitsignal);
	
	/* now the socket, so that the irc client may connect */
	if( (netfd = socket(AF_INET, SOCK_STREAM, 0)) == -1)
	{
		printf("%s\n","creating a socket failed!");
		exit(1);
	}
	
#if defined __CYGWIN__  /* cygwin */
	memset(&servaddr, 0, sizeof(servaddr));
#else                   /* cygwin */
	bzero(&servaddr, sizeof(servaddr));
#endif                  /* cygwin */

	servaddr.sin_family = AF_INET;
	servaddr.sin_addr.s_addr = htonl(INADDR_ANY);
	servaddr.sin_port = htons(cmd->port);

	setsockopt(netfd, SOL_SOCKET, SO_REUSEADDR, &on, sizeof(on));
	if( bind(netfd, (struct sockaddr *) &servaddr, sizeof(servaddr)) == -1)
	{
		printf("%s\n","bind failed!");
		puts(strerror(errno));
		exit(1);
	}
	
	printf("now connect with your IRC client to localhost:%d\n",cmd->port);
	listen(netfd, 10);
	
	clilen = sizeof(cliaddr);
	
	connfd = accept(netfd, (struct sockaddr *) &cliaddr, &clilen);

//	close(netfd); //FIXME

	printf("Foreign Host: %s\n", inet_ntoa(cliaddr.sin_addr));
/*
	printf("Would you like to allow this connect?\n");
	printf("\"y\" to allow\n");
	if(fgets(buff, 256, stdin)!=NULL)
	if(strcmp(buff,"y\n")==0) printf("allowed\n");
*/
	
	/* now connect to the server */
	if(Tcpstuff_Connect(SERV)<0)
	{
		printf("connect to server failed\nbye\n");
		exit(1);
	}

	/* this is needed for select */
	maxfdp1 = max(SERV->SOCKET,connfd) + 1;

	/* now lets get to select */
	FD_ZERO(&rset);
	for( ; ; )
	{
		FD_SET(fileno(stdin), &rset);
		FD_SET(connfd, &rset);
		FD_SET(SERV->SOCKET, &rset);

		select(maxfdp1,&rset,NULL,NULL,NULL);

		if(FD_ISSET(connfd,&rset))
		{
			socket_to_sockbuf();
			while(sockbuf_to_put_to_server()==0);
		}

		if(FD_ISSET(SERV->SOCKET,&rset))
		{
			Tcpstuff_socket_to_sockbuf(SERV);
			SERVbuf_to_clientput();
		}
	}
	exit(0);
}
