/* 
    SIT - SIS is tiny, intended to be a IRC Client for secure communication
	on top of the IRC protocol as described in RFC 1459
    Copyright (C) 2000-2002  Christian Leber

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    e-mail: christian@leber.de
    I hope this is GPL compliant, if not please contact me.
*/


#include "config.h"
#include <sys/types.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "dstruct.h"

/* ######################################################## */
/* for the NICKS                                            */

/* this returns a hash value for the given string, the
   string should have been set tolower allready...          */      
unsigned DS_nick_hash(char *s)
{
	unsigned hashval;
	
	for(hashval = 0; *s != '\0'; s++)
		hashval = *s + 31 * hashval;
	return hashval % DS_NICK_HASHSIZE;
}

/* this returns a pointer to the data structure of the      */
/* given nick                                               */
node_nick_list *DS_nick_lookup(node_s_list *this_DS_s, char *nick)
{
	node_nick_list *np;
	char *nick_tolower;
	nick_tolower = irc_tolower(nick);
	
	for(np=this_DS_s->DS_nick[DS_nick_hash(nick_tolower)]; np != NULL; np = np->next)
		if(strcmp(nick_tolower, np->nick) == 0)
		{
			free(nick_tolower);
			return np; /* do issa ja */
		}
	free(nick_tolower);
	return NULL; /* verdammt */
}

node_nick_list *DS_nick_install(node_s_list *this_DS_s, char *onick)
{
	node_nick_list *np;
	unsigned hashval;
	char *nick; /* set tolower, we point to this so we can't free it */
	
	if(this_DS_s==NULL) return NULL;

	nick = irc_tolower(onick);
		
	if((np=DS_nick_lookup(this_DS_s,nick)) == NULL)
	{
		np = (node_nick_list *)malloc(sizeof(*np));
		if(np==NULL || (np->nick = nick)==NULL)
		{
			return NULL;
		}
		hashval = DS_nick_hash(nick);
		np->next = this_DS_s->DS_nick[hashval];
		this_DS_s->DS_nick[hashval] = np;
	}
	else
		free(np->onick);
	if((np->onick = strdup(onick)) == NULL)
	{
		return NULL;
	}
	np->host = NULL;
	np->realname = NULL;
	np->username = NULL;

	np->query_mode = 0;
	
	/* ******************** */
	/* Crypto related stuff */
	/* normaly we do not want to send a symkey the next possible time */
	np->send_pers_symkey = 0;

	np->rsa_public = NULL;
	np->blowfish_query_send = NULL;
	np->blowfish_query_recv = NULL;

	return np;
}

void DS_nick_undef_destroy(node_nick_list *destroy)
{
	free(destroy->nick);
	free(destroy->onick);
	free(destroy);
}

int DS_nick_undef(node_s_list *this_DS_s, char *onick)
{
	/* onick because we take care in this function of low/high issue */
	node_nick_list *np, *save;
	unsigned hashval;
	char *nick;

	nick = irc_tolower(onick);

	hashval = DS_nick_hash(nick);

	if(this_DS_s->DS_nick[hashval]==NULL) return -1;

	if(strcmp(nick,this_DS_s->DS_nick[hashval]->nick)==0)
	{
		save=this_DS_s->DS_nick[hashval];
		this_DS_s->DS_nick[hashval]=save->next;
		DS_nick_undef_destroy(save);
		return 0;
	}
	else
	{	
		np=this_DS_s->DS_nick[hashval];
		while(np->next != NULL)
		{
			if(strcmp(nick, np->next->nick) == 0)
			{
				save=np->next;
				np->next=save->next;
				DS_nick_undef_destroy(save);
				return 0;
			}
			np = np->next;
		}
		return -1;
	}
}

void DS_nick_giveall(node_s_list *this_DS_s)
{
	int i;
	node_nick_list *np;
	
	printf("%s\n","#### DS_nick_giveall ####");
	for(i=0;i<DS_NICK_HASHSIZE;i++)
	{
		np = this_DS_s->DS_nick[i];
		while(np!=NULL)
		{
		printf("%d - %s - %s\n",i,np->nick,np->onick);
		np=np->next;
		}
	}	
}


/*#################################################################*/


char *irc_tolower(char *input)
{
	char *s, *output;
	
	output = strdup(input);
	for(s=output; *s != '\0'; s++)
		if( (*s>=65) && (*s<=93) ) *s = *s + 32;
	return output;
}

