/* 
    tcpstuff.c

    Copyright (C) 2000-2002  Christian Leber

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

    e-mail: christian@leber.de
    I hope this is GPL compliant, if not please contact me.
*/

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <string.h>
#include <strings.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#if __EMX__ || __CYGWIN__
	#include <netinet/in.h>
	#include <sys/socket.h>
	#include <strings.h>
#endif
#if __CYGWIN__
	#include <arpa/inet.h>  /* seems to be a problem with __EMX__ */
#endif
   
#include <string.h>
#include <netdb.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <signal.h>
#include <sys/wait.h>

#include "tcpstuff.h"

int Tcpstuff_init(struct_tcpstuff *this_DS,char *server, unsigned short int port)
{
	this_DS->SOCKET=0;
	this_DS->SERVER=strdup(server);
	this_DS->PORT=port;
	this_DS->SOCKBUF = (unsigned char *)malloc(2049);
	this_DS->SOCKBUF_POS = 0;
	this_DS->SOCKBUF_SIZE = 2048;

	return 0;
}

int Tcpstuff_socket_to_sockbuf(struct_tcpstuff *this_DS)
{
	int length;
	
	if(this_DS->SOCKBUF_SIZE <= this_DS->SOCKBUF_POS + 1)
		return 0;
	length = recv(this_DS->SOCKET,this_DS->SOCKBUF+this_DS->SOCKBUF_POS,
				this_DS->SOCKBUF_SIZE-this_DS->SOCKBUF_POS-1,0);
	if( length == -1)
	{
		return -1;
	}
	this_DS->SOCKBUF[this_DS->SOCKBUF_POS+length] = 0;
	this_DS->SOCKBUF_POS += length;
	return length;
}

int Tcpstuff_Connect(struct_tcpstuff *this_DS)
{
#if !__EMX__ && !__CYGWIN__
	/* ok... here it's all right, we're NOT on an OS/2 or cygwin system*/
	struct sockaddr *servaddr;
	socklen_t len;
	char port_str[10];
	
	if(this_DS->SOCKET != 0) close(this_DS->SOCKET);
	this_DS->SOCKET = 0;

	this_DS->SOCKBUF_POS = 0;
	this_DS->SOCKBUF[0] = 0;
  
  	snprintf(port_str,9,"%u",this_DS->PORT);
  
	this_DS->SOCKET = Tcpstuff_Get_Socket(this_DS->SERVER, port_str);

	if(this_DS->SOCKET == -1) return -1;

	servaddr = malloc(128);
	len = 128;
	
	getpeername(this_DS->SOCKET, servaddr, &len);

/*	printf("CONNECTED: %s\n", sock_ntop_host(servaddr,len));*/

	return 0;

#else
	/* here all is lost, it's... OS half */
	/* or the poor unix emulation on a win32 system... */
	struct sockaddr_in servaddr;
	struct hostent *HostAddr;
	
	if(this_DS->SOCKET != 0) close(this_DS->SOCKET);
	this_DS->SOCKET = 0;

	this_DS->SOCKBUF_POS = 0;
	this_DS->SOCKBUF[0] = 0;
  
	this_DS->SOCKET = socket(AF_INET, SOCK_STREAM, getprotobyname("tcp")->p_proto);
	if(this_DS->SOCKET == -1) return -1;

	memset(&servaddr, 0, sizeof(servaddr));

	servaddr.sin_family = AF_INET;
	servaddr.sin_port = htons(this_DS->PORT);
	
	if(inet_addr(this_DS->SERVER) != INADDR_NONE) 
	{
		servaddr.sin_addr.s_addr = inet_addr(this_DS->SERVER);
	}
	else
	{
		HostAddr = (struct hostent *)gethostbyname(this_DS->SERVER);
		if(HostAddr == NULL)
		{
			printf("DEBUG: %s\n","SIT_Connect; gethostbyname() failed");
			return -1;
		}
		memcpy((void *)&servaddr.sin_addr, HostAddr->h_addr, HostAddr->h_length);
	}
	if(connect(this_DS->SOCKET, (struct sockaddr *)&servaddr, sizeof(servaddr)) == -1)
		return -1;
	printf("DEBUG: %s\n","SIT_Connect: connected");
	return 0;

#endif
}

void Tcpstuff_Close(struct_tcpstuff *this_DS)
{
	close(this_DS->SOCKET);
	this_DS->SOCKET = 0;
}

#if !__EMX__ && !__CYGWIN__
int Tcpstuff_Get_Socket(char *host, char *port)
{
	int output_socket; /* we'll gibe this back */
	int valuu;
	struct addrinfo *hints, *res, *ressave;

	hints = malloc(sizeof(struct addrinfo));
	bzero(hints, sizeof(struct addrinfo));
	
	hints->ai_family = AF_UNSPEC;
	hints->ai_socktype = SOCK_STREAM;
	
	if( ( valuu = getaddrinfo(host, port, hints, &res)) != 0)
	{
		printf("Tcpstuff_Get_Socket: %s\n",	gai_strerror(valuu));
		exit(1);
	}
	ressave=res;
	do
	{
		output_socket = socket(res->ai_family, res->ai_socktype, res->ai_protocol);
		if(output_socket < 0) continue;
		if(connect(output_socket, res->ai_addr, res->ai_addrlen) == 0) break;
		close(output_socket);
	} while( ( res=res->ai_next) != NULL);
	
	if(res==NULL)
	{
		printf("fatal error: Can't connect to %s:%s!\n",host,port);
		exit(1);
	}
	freeaddrinfo(ressave);

	return (output_socket);	
}
#endif
